#version 130
precision highp float;

uniform sampler2D colorTexture;
//uniform sampler2DShadow depthTexture;
uniform sampler2D depthTexture;
uniform sampler2D mbInfoTexture;
uniform mat4 viewProjectionInverse;
uniform mat4 lastViewProjection;
uniform int blurSampleCount;

in vec2 frag_texCoord;

out vec4 fragColor;

/*****
** code aus 
** GPU Gems 3 Chapter 27. Motion Blur as a Post-Processing Effect
** von HLSL/Cg nach GLSL portiert und an meine "engine" angepast
*/

vec2 getVelocity(float yZoverW)
{
	// ### ---  Extracts the Per-Pixel World-Space Positions of the Objects That Were Rendered to the Depth Buffer
	// Get the depth buffer value at this pixel.  
	//float zOverW = shadow2D(depthTexture, frag_texCoord);  
	//float zOverW = (texture2D(depthTexture, frag_texCoord)).r;  
	// H is the viewport position at this pixel in the range -1 to 1.  
	vec4 H = vec4(frag_texCoord.x * 2.f - 1.0f, (1.0f - frag_texCoord.y) * 2.0f - 1.0f,  yZoverW, 1.0f);  
	// Transform by the view-projection inverse.  
	vec4 D = viewProjectionInverse * H;  
	// Divide by w to get the world position.  
	vec4 worldPos = D / D.w;  


	// ### --- Computes the Per-Pixel Velocity Vectors That Determine the Direction to Blur the Image
	// Current viewport position
	vec4 currentPos = H;
	// Use the world position, and transform by the previous view-  
	// projection matrix.  
	vec4 previousPos = lastViewProjection * worldPos;  
	// Convert to nonhomogeneous points [-1,1] by dividing by w.  
	previousPos /= previousPos.w;  
	// Use this frame's position and last frame's to compute the pixel  
	// velocity.  
	vec2 velosity = ((currentPos - previousPos)/20.0f).xy;
	return vec2(velosity.x * -1, velosity.y);
}

void main()
{
	//fragColor = texture2D(colorTexture, frag_texCoord);
	//fragColor = texture2D(colorTexture, frag_texCoord)-1.0f;
	
	//fragColor = texture2D(mbInfoTexture, frag_texCoord);
	//fragColor = vec4(0.0f, 0.0f, fragColor.a, 1.0f);
	
	//fragColor = texture2D(mbInfoTexture, frag_texCoord);
	//fragColor = fragColor*20.0f - 19.2f;
	
	/*
	// visualisiere die depth map
	//float depth = (texture2D(depthTexture, frag_texCoord)).r;
	float depth = (textureOffset(depthTexture, frag_texCoord, ivec2(-2, 2))).r;
	depth = depth*20.0f - 19.2f;
	fragColor = vec4(depth, depth, depth, 1.0f);
	*/
	
	/*
	// visualisiert die bewegungstrke (weiser => mehr bewegung)
	float zOverW = (texture2D(depthTexture, frag_texCoord)).r;
	vec2 velocity = getVelocity(zOverW);
	float velocityABS = length(velocity) * 30.0;
	fragColor = vec4(velocityABS, velocityABS, velocityABS, 1.0f);
	*/
	
	
	// ### ---  Uses the Velocity Vector at the Current Pixel to Sample the Color Buffer Multiple Times to Achieve the Motion Blur Effect
	// Get the initial color at this pixel.  
	vec4 color = texture2D(colorTexture, frag_texCoord);
	float zOverW = (texture2D(depthTexture, frag_texCoord)).r; 
	int localSampleCount = 1;
	if(!(texture2D(mbInfoTexture, frag_texCoord).a > 0.0f))
	{
		vec2 velocity = getVelocity(zOverW);
		vec2 texCoord = frag_texCoord + velocity;  
		for(int i = 1; i < blurSampleCount; ++i, texCoord += velocity)  
		{
			if(!(texture2D(mbInfoTexture, texCoord).a > 0.0f))
			{
				// Sample the color buffer along the velocity vector.  
				vec4 currentColor = texture2D(colorTexture, texCoord);  
				// Add the current color to our color sum.  
				color += currentColor;
				
				localSampleCount++;
			}
		}
		// Average all of the samples to get the final blur color.  
		fragColor = color / localSampleCount;
	}
	else // nicht blurn
	{
		fragColor = color;
	}
	
	gl_FragDepth = zOverW;
	
	/////
	//float depth = zOverW*20.0f - 19.2f;
	//fragColor = vec4(depth, depth, depth, 1.0f);
}