#version 130
precision highp float;


uniform sampler2D texture_color;
uniform sampler2D texture_normal;

uniform vec4 diffuseColor;
uniform vec4 specColor;
uniform float ref;
uniform float spec;
uniform float hard;
uniform float alpha;
uniform float emit;

uniform vec3 collisionPoints[30];
uniform float collisionIntense[30];
uniform bool depthMask;


in vec2 frag_texCoord;
in vec3 frag_eyeVector;
in vec3 frag_lightVector[5];
in vec3 frag_normal;

in vec3 frag_pos;


out vec4 fragColor;
out vec4 mbControll;


const float maxColliDis = 0.4f;

vec4 collisionColor(vec4 yColor, vec4 yTexel)
{
	float alpha = 0.0f;
	float texelBrightnes = length(yTexel.xy)*0.6f; // texelBrightnes brauch ich damit die eher weisen bereiche weiter reichen als die maxColliDis und die ganze sache "ausfranst"
	for (int i = 15; i < 30; i++)
	{
		float distanc = abs(length(collisionPoints[i] - frag_pos));
		float texelBrightnesFaktor = texelBrightnes * collisionIntense[i]; //=> damit sich die plize langsm zurckziehen und nicht blos transparenter werden
		if((distanc - texelBrightnesFaktor) < maxColliDis && collisionIntense[i] > 0.1f)
		{
			alpha += ((maxColliDis - distanc + texelBrightnesFaktor) / (maxColliDis)) * collisionIntense[i];
		}
	}
	//alpha = min(alpha, 1.0f); // nicht grer als 1.0f
	yColor = yColor + yColor * (alpha * 15.0f); // * texelBrightnes; // aufhellen des treffer punktes
	
	// Damit die blitze blauer sind:
	yColor = yColor * vec4(0.5f, 0.5f, 1.0f, 1.0f);
	
	return yColor;
}

void main()
{
	if(!depthMask)
	{
		vec3 normal = (texture2D(texture_normal, frag_texCoord.st).rgb * 2.0f - vec3(1.0f, 1.0f, 1.0f)) * 1.0f;
		float diffuse = 0.0f;
		float specular = 0.0f;
		
		for (int i = 0; i < 5; i++)
		{
			vec3 lightVector = normalize(frag_lightVector[i]);
			
			diffuse += max(0.0f, dot(normal, lightVector));
			
			if (dot(normal, lightVector) > 0)
			{
				specular += pow(max(0.0f, dot(reflect(-lightVector, normal), normalize(frag_eyeVector))), hard);
			}
		}
		
		//diffuse = max(0.2f, min(1.0f, diffuse * ref + emit));
		//specular = min(1.0f, specular * spec);
		diffuse = diffuse * ref + emit;
		specular = specular * spec;
		float alpha4glow = diffuse - 1.0f; // + specular - 1.0f;
		
		// HDR on - next lines off
		//diffuse = max(0.2f, min(1.0f, diffuse));
		//specular = min(1.0f, specular);
		
		vec4 texel = texture2D(texture_color, frag_texCoord.st);
		fragColor = texel * diffuse + specColor * specular;
		
		alpha4glow += alpha;
		fragColor = vec4(fragColor.rgb, alpha4glow);
		
		//-------------------------------
		
		fragColor.a = emit;
		fragColor = collisionColor(fragColor, texel);
		
		mbControll = vec4(gl_FragCoord.z, 0.0f, 0.0f, 1.0f);
	}
	else
	{
		fragColor = vec4(0.0f, 0.0f, 0.0f, 0.0f);
	}
}