#version 130
precision highp float;

uniform mat4 persp;
uniform mat4 camera;
uniform mat4 model;

uniform vec3 lightPosition[5];

in vec4 vertex;
in vec3 normal;
in vec2 texCoord;
in vec3 tangent;


out vec2 frag_texCoord;
out vec3 frag_eyeVector;
out vec3 frag_lightVector[5];

out vec3 frag_normal;
out vec3 frag_pos;

void main()
{	
	//Position berechnen
	
	vec4 wpos = model * vertex;
	vec4 pos = camera * model * vertex;
	
	vec3 lightDirection[5];
	
	for (int i = 0; i < 5; i ++)
	{
		vec4 lpos = camera * vec4(lightPosition[i], 1.0f);
		lightDirection[i] = lpos.xyz - pos.xyz;
	}
	
	//Alles in den Tangentenraum transformieren
	
	//Tangentenraum berechnen
	vec3 n = (camera * model * vec4(normal, 0.0f)).xyz;
	vec3 t = (camera * model * vec4(tangent, 0.0f)).xyz;
	vec3 b = cross(t, n);
	
	for (int i = 0; i < 5; i++)
	{
		frag_lightVector[i].x = dot(lightDirection[i], t);
		frag_lightVector[i].y = dot(lightDirection[i], b);
		frag_lightVector[i].z = dot(lightDirection[i], n);
	}
	
	frag_eyeVector.x = dot(-(pos.xyz), t);
	frag_eyeVector.y = dot(-(pos.xyz), b);
	frag_eyeVector.z = dot(-(pos.xyz), n);
	
	frag_texCoord = texCoord;
	
	/*frag_lightVector = lightDirection;
	frag_eyeVector = -pos.xyz;*/
	frag_normal = (camera * model * vec4(normal, 0.0f)).xyz;
	
	gl_Position = persp * camera * model * vertex;	
	
	//frag_pos = (model * vertex).xyz;
	frag_pos = vertex.xyz;
}




/*in vec4 vertex;
in vec3 normal;
in vec2 texCoord;

out vec4 world_position;
out vec3 world_normal;
out vec2 world_texCoord;

void main()
{
	vec4 norm4 = vec4(normal, 0.0f);
	world_normal = (model * norm4).xyz;
	
	world_position = model * vertex;
	world_texCoord = texCoord;
	
	gl_Position = persp * camera * model * vertex;
}*/